/*
 * Licensed under the GNU Lesser General Public License Version 3
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the license, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

// generated automatically - do not change


module appstream.Release;

private import appstream.Artifact;
private import appstream.Checksum;
private import appstream.Issue;
private import appstream.c.functions;
public  import appstream.c.types;
private import glib.ConstructionException;
private import glib.PtrArray;
private import glib.Str;
private import glib.c.functions;
private import gobject.ObjectG;


/** */
public class Release : ObjectG
{
	/** the main Gtk struct */
	protected AsRelease* asRelease;

	/** Get the main Gtk struct */
	public AsRelease* getReleaseStruct(bool transferOwnership = false)
	{
		if (transferOwnership)
			ownedRef = false;
		return asRelease;
	}

	/** the main Gtk struct as a void* */
	protected override void* getStruct()
	{
		return cast(void*)asRelease;
	}

	/**
	 * Sets our main struct and passes it to the parent class.
	 */
	public this (AsRelease* asRelease, bool ownedRef = false)
	{
		this.asRelease = asRelease;
		super(cast(GObject*)asRelease, ownedRef);
	}


	/** */
	public static GType getType()
	{
		return as_release_get_type();
	}

	/**
	 * Creates a new #AsRelease.
	 *
	 * Returns: a #AsRelease
	 *
	 * Throws: ConstructionException GTK+ fails to create the object.
	 */
	public this()
	{
		auto __p = as_release_new();

		if(__p is null)
		{
			throw new ConstructionException("null returned by new");
		}

		this(cast(AsRelease*) __p, true);
	}

	/**
	 * Add an artifact (binary / source download) for this release.
	 *
	 * Params:
	 *     artifact = The #AsArtifact.
	 *
	 * Since: 0.12.6
	 */
	public void addArtifact(Artifact artifact)
	{
		as_release_add_artifact(asRelease, (artifact is null) ? null : artifact.getArtifactStruct());
	}

	/**
	 * Add a checksum for the file associated with this release.
	 *
	 * Deprecated: Use the #AsArtifact directly to obtain this information.
	 *
	 * Params:
	 *     cs = The #AsChecksum.
	 *
	 * Since: 0.8.2
	 */
	public void addChecksum(Checksum cs)
	{
		as_release_add_checksum(asRelease, (cs is null) ? null : cs.getChecksumStruct());
	}

	/**
	 * Add information about a (resolved) issue to this release.
	 *
	 * Params:
	 *     issue = The #AsIssue.
	 *
	 * Since: 0.12.9
	 */
	public void addIssue(Issue issue)
	{
		as_release_add_issue(asRelease, (issue is null) ? null : issue.getIssueStruct());
	}

	/**
	 * Adds a release location.
	 *
	 * Deprecated: Use the #AsArtifact directly to obtain this information.
	 *
	 * Params:
	 *     location = An URL of the download location
	 *
	 * Since: 0.8.1
	 */
	public void addLocation(string location)
	{
		as_release_add_location(asRelease, Str.toStringz(location));
	}

	/**
	 * Get the current active locale, which
	 * is used to get localized messages.
	 */
	public string getActiveLocale()
	{
		return Str.toString(as_release_get_active_locale(asRelease));
	}

	/**
	 * Get a list of all downloadable artifacts that are associated with
	 * this release.
	 *
	 * Returns: an array of #AsArtifact objects.
	 *
	 * Since: 0.12.6
	 */
	public PtrArray getArtifacts()
	{
		auto __p = as_release_get_artifacts(asRelease);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Gets the release checksum
	 *
	 * Deprecated: Use the #AsArtifact directly to obtain this information.
	 *
	 * Returns: an #AsChecksum, or %NULL for not set or invalid
	 *
	 * Since: 0.8.2
	 */
	public Checksum getChecksum(AsChecksumKind kind)
	{
		auto __p = as_release_get_checksum(asRelease, kind);

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(Checksum)(cast(AsChecksum*) __p);
	}

	/**
	 * Get a list of all checksums we have for this release.
	 *
	 * Deprecated: Use the #AsArtifact directly to obtain this information.
	 *
	 * Returns: an array of #AsChecksum objects.
	 *
	 * Since: 0.10
	 */
	public PtrArray getChecksums()
	{
		auto __p = as_release_get_checksums(asRelease);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Gets the release date.
	 *
	 * Returns: The date in ISO8601 format.
	 *
	 * Since: 0.12.5
	 */
	public string getDate()
	{
		return Str.toString(as_release_get_date(asRelease));
	}

	/**
	 * Gets the end-of-life date for this release.
	 *
	 * Returns: The EOL date in ISO8601 format.
	 *
	 * Since: 0.12.5
	 */
	public string getDateEol()
	{
		return Str.toString(as_release_get_date_eol(asRelease));
	}

	/**
	 * Gets the release description markup for a given locale.
	 *
	 * Returns: markup, or %NULL for not set or invalid
	 */
	public string getDescription()
	{
		return Str.toString(as_release_get_description(asRelease));
	}

	/**
	 * Get a list of all issues resolved by this release.
	 *
	 * Returns: an array of #AsIssue objects.
	 *
	 * Since: 0.12.9
	 */
	public PtrArray getIssues()
	{
		auto __p = as_release_get_issues(asRelease);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Gets the type of the release.
	 * (development or stable release)
	 *
	 * Since: 0.12.0
	 */
	public AsReleaseKind getKind()
	{
		return as_release_get_kind(asRelease);
	}

	/**
	 * Gets the release locations, typically URLs.
	 *
	 * Deprecated: Use the #AsArtifact directly to obtain this information.
	 *
	 * Returns: list of locations
	 *
	 * Since: 0.8.1
	 */
	public PtrArray getLocations()
	{
		auto __p = as_release_get_locations(asRelease);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Gets the release size.
	 *
	 * Deprecated: Use the #AsArtifact directly to obtain this information.
	 *
	 * Params:
	 *     kind = a #AsSizeKind
	 *
	 * Returns: The size of the given kind of this release.
	 *
	 * Since: 0.8.6
	 */
	public ulong getSize(AsSizeKind kind)
	{
		return as_release_get_size(asRelease, kind);
	}

	/**
	 * Gets the release timestamp.
	 *
	 * Returns: timestamp, or 0 for unset
	 */
	public ulong getTimestamp()
	{
		return as_release_get_timestamp(asRelease);
	}

	/**
	 * Gets the UNIX timestamp for the date when this
	 * release is out of support (end-of-life).
	 *
	 * Returns: UNIX timestamp, or 0 for unset or invalid.
	 *
	 * Since: 0.12.5
	 */
	public ulong getTimestampEol()
	{
		return as_release_get_timestamp_eol(asRelease);
	}

	/**
	 * Gets the urgency of the release
	 * (showing how important it is to update to a more recent release)
	 *
	 * Returns: #AsUrgencyKind, or %AS_URGENCY_KIND_UNKNOWN for not set
	 *
	 * Since: 0.6.5
	 */
	public AsUrgencyKind getUrgency()
	{
		return as_release_get_urgency(asRelease);
	}

	/**
	 * Gets an URL.
	 *
	 * Params:
	 *     urlKind = the URL kind, e.g. %AS_RELEASE_URL_KIND_DETAILS.
	 *
	 * Returns: string, or %NULL if unset
	 *
	 * Since: 0.12.5
	 */
	public string getUrl(AsReleaseUrlKind urlKind)
	{
		return Str.toString(as_release_get_url(asRelease, urlKind));
	}

	/**
	 * Gets the release version.
	 *
	 * Returns: string, or %NULL for not set or invalid
	 */
	public string getVersion()
	{
		return Str.toString(as_release_get_version(asRelease));
	}

	/**
	 * Set the current active locale, which
	 * is used to get localized messages.
	 * If the #AsComponent linking this #AsRelease was fetched
	 * from a localized database, usually only
	 * one locale is available.
	 */
	public void setActiveLocale(string locale)
	{
		as_release_set_active_locale(asRelease, Str.toStringz(locale));
	}

	/**
	 * Sets the release date.
	 *
	 * Params:
	 *     date = the date in ISO8601 format.
	 *
	 * Since: 0.12.5
	 */
	public void setDate(string date)
	{
		as_release_set_date(asRelease, Str.toStringz(date));
	}

	/**
	 * Sets the end-of-life date for this release.
	 *
	 * Params:
	 *     date = the EOL date in ISO8601 format.
	 *
	 * Since: 0.12.5
	 */
	public void setDateEol(string date)
	{
		as_release_set_date_eol(asRelease, Str.toStringz(date));
	}

	/**
	 * Sets the description release markup.
	 *
	 * Params:
	 *     description = the description markup.
	 */
	public void setDescription(string description, string locale)
	{
		as_release_set_description(asRelease, Str.toStringz(description), Str.toStringz(locale));
	}

	/**
	 * Sets the release kind to distinguish between end-user ready
	 * stable releases and development prereleases..
	 *
	 * Params:
	 *     kind = the #AsReleaseKind
	 *
	 * Since: 0.12.0
	 */
	public void setKind(AsReleaseKind kind)
	{
		as_release_set_kind(asRelease, kind);
	}

	/**
	 * Sets the release size for the given kind.
	 *
	 * Deprecated: Use the #AsArtifact directly to obtain this information.
	 *
	 * Params:
	 *     size = a size in bytes, or 0 for unknown
	 *     kind = a #AsSizeKind
	 *
	 * Since: 0.8.6
	 */
	public void setSize(ulong size, AsSizeKind kind)
	{
		as_release_set_size(asRelease, size, kind);
	}

	/**
	 * Sets the release timestamp.
	 *
	 * Params:
	 *     timestamp = the timestamp value.
	 */
	public void setTimestamp(ulong timestamp)
	{
		as_release_set_timestamp(asRelease, timestamp);
	}

	/**
	 * Sets the UNIX timestamp for the date when this
	 * release is out of support (end-of-life).
	 *
	 * Params:
	 *     timestamp = the timestamp value.
	 *
	 * Since: 0.12.5
	 */
	public void setTimestampEol(ulong timestamp)
	{
		as_release_set_timestamp_eol(asRelease, timestamp);
	}

	/**
	 * Sets the release urgency.
	 *
	 * Params:
	 *     urgency = the urgency of this release/update (as #AsUrgencyKind)
	 *
	 * Since: 0.6.5
	 */
	public void setUrgency(AsUrgencyKind urgency)
	{
		as_release_set_urgency(asRelease, urgency);
	}

	/**
	 * Sets an URL for this release.
	 *
	 * Params:
	 *     urlKind = the URL kind, e.g. %AS_RELEASE_URL_KIND_DETAILS
	 *     url = the full URL.
	 *
	 * Since: 0.12.5
	 */
	public void setUrl(AsReleaseUrlKind urlKind, string url)
	{
		as_release_set_url(asRelease, urlKind, Str.toStringz(url));
	}

	/**
	 * Sets the release version.
	 *
	 * Params:
	 *     version_ = the version string.
	 */
	public void setVersion(string version_)
	{
		as_release_set_version(asRelease, Str.toStringz(version_));
	}

	/**
	 * Compare the version numbers of two releases.
	 *
	 * Params:
	 *     rel2 = an #AsRelease
	 *
	 * Returns: 1 if @rel1 version is higher than @rel2, 0 if versions are equal, -1 if @rel2 version is higher than @rel1.
	 */
	public int vercmp(Release rel2)
	{
		return as_release_vercmp(asRelease, (rel2 is null) ? null : rel2.getReleaseStruct());
	}
}
