/*
 * Licensed under the GNU Lesser General Public License Version 3
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the license, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

// generated automatically - do not change


module appstream.Validator;

private import appstream.c.functions;
public  import appstream.c.types;
private import gio.FileIF;
private import glib.Bytes;
private import glib.ConstructionException;
private import glib.HashTable;
private import glib.ListG;
private import glib.Str;
private import glib.c.functions;
private import gobject.ObjectG;


/** */
public class Validator : ObjectG
{
	/** the main Gtk struct */
	protected AsValidator* asValidator;

	/** Get the main Gtk struct */
	public AsValidator* getValidatorStruct(bool transferOwnership = false)
	{
		if (transferOwnership)
			ownedRef = false;
		return asValidator;
	}

	/** the main Gtk struct as a void* */
	protected override void* getStruct()
	{
		return cast(void*)asValidator;
	}

	/**
	 * Sets our main struct and passes it to the parent class.
	 */
	public this (AsValidator* asValidator, bool ownedRef = false)
	{
		this.asValidator = asValidator;
		super(cast(GObject*)asValidator, ownedRef);
	}


	/** */
	public static GType getType()
	{
		return as_validator_get_type();
	}

	/**
	 * Creates a new #AsValidator.
	 *
	 * Returns: an #AsValidator
	 *
	 * Throws: ConstructionException GTK+ fails to create the object.
	 */
	public this()
	{
		auto __p = as_validator_new();

		if(__p is null)
		{
			throw new ConstructionException("null returned by new");
		}

		this(cast(AsValidator*) __p, true);
	}

	/**
	 * Clears the list of issues
	 */
	public void clearIssues()
	{
		as_validator_clear_issues(asValidator);
	}

	/**
	 * Returns: %TRUE in case we check if remote URLs exist.
	 */
	public bool getCheckUrls()
	{
		return as_validator_get_check_urls(asValidator) != 0;
	}

	/**
	 * Get a list of found metadata format issues.
	 *
	 * Returns: a list of #AsValidatorIssue instances, free with g_list_free()
	 */
	public ListG getIssues()
	{
		auto __p = as_validator_get_issues(asValidator);

		if(__p is null)
		{
			return null;
		}

		return new ListG(cast(GList*) __p);
	}

	/**
	 * Get a hash table of filenames mapped to lists of issues.
	 * This is useful if validation was requested for multiple files and
	 * a list of issues per-file is desired without prior explicit sorting.
	 *
	 * Returns: a file to issue list mapping
	 *
	 * Since: 0.12.8
	 */
	public HashTable getIssuesPerFile()
	{
		auto __p = as_validator_get_issues_per_file(asValidator);

		if(__p is null)
		{
			return null;
		}

		return new HashTable(cast(GHashTable*) __p);
	}

	/** */
	public bool getReportYaml(string[] yamlReport)
	{
		return as_validator_get_report_yaml(asValidator, Str.toStringzArray(yamlReport)) != 0;
	}

	/**
	 * Get the explanatory text for a given issue tag.
	 *
	 * Returns: Explanation text.
	 */
	public string getTagExplanation(string tag)
	{
		return Str.toString(as_validator_get_tag_explanation(asValidator, Str.toStringz(tag)));
	}

	/**
	 * Get the severity for a given issue tag.
	 *
	 * Returns: The #AsIssueSeverity
	 */
	public AsIssueSeverity getTagSeverity(string tag)
	{
		return as_validator_get_tag_severity(asValidator, Str.toStringz(tag));
	}

	/**
	 * Get an array of all tags known to the validator.
	 *
	 * Returns: A string array of tags
	 */
	public string[] getTags()
	{
		auto retStr = as_validator_get_tags(asValidator);

		scope(exit) Str.freeStringArray(retStr);
		return Str.toStringArray(retStr);
	}

	/**
	 * Set this value to make the #AsValidator check whether remote URLs
	 * actually exist.
	 */
	public void setCheckUrls(bool value)
	{
		as_validator_set_check_urls(asValidator, value);
	}

	/**
	 * Validate AppStream XML data from a byte array.
	 * Remember to run %as_validator_clear_issues if you do not want previous
	 * validation runs to affect the outcome of this validation.
	 *
	 * Params:
	 *     metadata = XML metadata as #GBytes.
	 *
	 * Returns: %TRUE if bytes validated successfully.
	 *
	 * Since: 0.14.0
	 */
	public bool validateBytes(Bytes metadata)
	{
		return as_validator_validate_bytes(asValidator, (metadata is null) ? null : metadata.getBytesStruct()) != 0;
	}

	/**
	 * Validate AppStream XML data.
	 * Remember to run %as_validator_clear_issues if you do not want previous
	 * validation runs to affect the outcome of this validation.
	 *
	 * Params:
	 *     metadata = XML metadata.
	 *
	 * Returns: %TRUE if data validated successfully.
	 */
	public bool validateData(string metadata)
	{
		return as_validator_validate_data(asValidator, Str.toStringz(metadata)) != 0;
	}

	/**
	 * Validate an AppStream XML file.
	 * Remember to run %as_validator_clear_issues if you do not want previous
	 * validation runs to affect the outcome of this validation.
	 *
	 * Params:
	 *     metadataFile = An AppStream XML file.
	 *
	 * Returns: %TRUE if file validated successfully.
	 */
	public bool validateFile(FileIF metadataFile)
	{
		return as_validator_validate_file(asValidator, (metadataFile is null) ? null : metadataFile.getFileStruct()) != 0;
	}

	/**
	 * Validate a full directory tree for issues in AppStream metadata.
	 *
	 * Params:
	 *     rootDir = The root directory of the filesystem tree that should be validated.
	 *
	 * Returns: %TRUE if file validated successfully.
	 */
	public bool validateTree(string rootDir)
	{
		return as_validator_validate_tree(asValidator, Str.toStringz(rootDir)) != 0;
	}
}
