/*
 * Licensed under the GNU Lesser General Public License Version 3
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the license, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

// generated automatically - do not change


module ascompose.Compose;

private import ascompose.IconPolicy;
private import ascompose.Result;
private import ascompose.Unit;
private import ascompose.c.functions;
public  import ascompose.c.types;
private import gio.Cancellable;
private import glib.ConstructionException;
private import glib.ErrorG;
private import glib.GException;
private import glib.PtrArray;
private import glib.Str;
private import glib.c.functions;
private import gobject.ObjectG;


/** */
public class Compose : ObjectG
{
	/** the main Gtk struct */
	protected AscCompose* ascCompose;

	/** Get the main Gtk struct */
	public AscCompose* getComposeStruct(bool transferOwnership = false)
	{
		if (transferOwnership)
			ownedRef = false;
		return ascCompose;
	}

	/** the main Gtk struct as a void* */
	protected override void* getStruct()
	{
		return cast(void*)ascCompose;
	}

	/**
	 * Sets our main struct and passes it to the parent class.
	 */
	public this (AscCompose* ascCompose, bool ownedRef = false)
	{
		this.ascCompose = ascCompose;
		super(cast(GObject*)ascCompose, ownedRef);
	}


	/** */
	public static GType getType()
	{
		return asc_compose_get_type();
	}

	/**
	 * Creates a new #AscCompose.
	 *
	 * Throws: ConstructionException GTK+ fails to create the object.
	 */
	public this()
	{
		auto __p = asc_compose_new();

		if(__p is null)
		{
			throw new ConstructionException("null returned by new");
		}

		this(cast(AscCompose*) __p, true);
	}

	/**
	 * Adds a component ID to the allowlist. If the list is not empty, only
	 * components in the list will be added to the metadata output.
	 *
	 * Params:
	 *     componentId = The component-id to whitelist
	 */
	public void addAllowedCid(string componentId)
	{
		asc_compose_add_allowed_cid(ascCompose, Str.toStringz(componentId));
	}

	/**
	 * Add a key to the allowlist that is used to filter custom tag values.
	 *
	 * Params:
	 *     keyId = the custom key to add to the allowed list.
	 */
	public void addCustomAllowed(string keyId)
	{
		asc_compose_add_custom_allowed(ascCompose, Str.toStringz(keyId));
	}

	/**
	 * Add compose flags.
	 *
	 * Params:
	 *     flags = The compose flags to add.
	 */
	public void addFlags(AscComposeFlags flags)
	{
		asc_compose_add_flags(ascCompose, flags);
	}

	/**
	 * Add an #AscUnit as data source for metadata processing.
	 *
	 * Params:
	 *     unit = The #AscUnit to add
	 */
	public void addUnit(Unit unit)
	{
		asc_compose_add_unit(ascCompose, (unit is null) ? null : unit.getUnitStruct());
	}

	/**
	 * Get the results components extracted in the last data processing run.
	 *
	 * Returns: The components
	 */
	public PtrArray fetchComponents()
	{
		auto __p = asc_compose_fetch_components(ascCompose);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Perform final validation of generated data for the specified
	 * result container.
	 *
	 * Params:
	 *     result = the #AscResult to finalize
	 */
	public void finalizeResult(Result result)
	{
		asc_compose_finalize_result(ascCompose, (result is null) ? null : result.getResultStruct());
	}

	/**
	 * Perform final validation of generated data.
	 * Calling this function is not necessary, unless the final check was explicitly
	 * disabled using the %ASC_COMPOSE_FLAG_NO_FINAL_CHECK flag.
	 */
	public void finalizeResults()
	{
		asc_compose_finalize_results(ascCompose);
	}

	/**
	 * Get the CA file used to verify peers with, or %NULL for default.
	 */
	public string getCainfo()
	{
		return Str.toString(asc_compose_get_cainfo(ascCompose));
	}

	/**
	 * Get the data result directory.
	 */
	public string getDataResultDir()
	{
		return Str.toString(asc_compose_get_data_result_dir(ascCompose));
	}

	/**
	 * Get the flags controlling compose behavior.
	 */
	public AscComposeFlags getFlags()
	{
		return asc_compose_get_flags(ascCompose);
	}

	/**
	 * get the format type we are generating.
	 */
	public AsFormatKind getFormat()
	{
		return asc_compose_get_format(ascCompose);
	}

	/**
	 * Get hints report output directory.
	 */
	public string getHintsResultDir()
	{
		return Str.toString(asc_compose_get_hints_result_dir(ascCompose));
	}

	/**
	 * Get the policy for how icons should be distributed to
	 * any AppStream clients.
	 *
	 * Returns: an #AscIconPolicy
	 */
	public IconPolicy getIconPolicy()
	{
		auto __p = asc_compose_get_icon_policy(ascCompose);

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(IconPolicy)(cast(AscIconPolicy*) __p);
	}

	/**
	 * Get the icon result directory.
	 */
	public string getIconsResultDir()
	{
		return Str.toString(asc_compose_get_icons_result_dir(ascCompose));
	}

	/**
	 * Get the unit we use for locale processing
	 *
	 * Returns: The unit used for locale processing, or %NULL for default.
	 */
	public Unit getLocaleUnit()
	{
		auto __p = asc_compose_get_locale_unit(ascCompose);

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(Unit)(cast(AscUnit*) __p);
	}

	/**
	 * Get the maximum size a screenshot video or image can have.
	 * A size < 0 may be returned for no limit, setting a limit of 0
	 * will disable screenshots.
	 */
	public ptrdiff_t getMaxScreenshotSize()
	{
		return asc_compose_get_max_screenshot_size(ascCompose);
	}

	/**
	 * Get the media base URL to be used for the generated data,
	 * or %NULL if this feature is not used.
	 */
	public string getMediaBaseurl()
	{
		return Str.toString(asc_compose_get_media_baseurl(ascCompose));
	}

	/**
	 * Get the media result directory, that can be served on a webserver.
	 */
	public string getMediaResultDir()
	{
		return Str.toString(asc_compose_get_media_result_dir(ascCompose));
	}

	/**
	 * Get the metadata origin field.
	 */
	public string getOrigin()
	{
		return Str.toString(asc_compose_get_origin(ascCompose));
	}

	/**
	 * Get the directory prefix used for processing.
	 */
	public string getPrefix()
	{
		return Str.toString(asc_compose_get_prefix(ascCompose));
	}

	/**
	 * Get the results of the last processing run.
	 *
	 * Returns: The results
	 */
	public PtrArray getResults()
	{
		auto __p = asc_compose_get_results(ascCompose);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Check if the last run generated any errors (which will cause metadata to be ignored).
	 *
	 * Returns: %TRUE if we had errors.
	 */
	public bool hasErrors()
	{
		return asc_compose_has_errors(ascCompose) != 0;
	}

	/**
	 * Remove a key from the allowlist used to filter the `<custom/>` tag entries.
	 *
	 * Params:
	 *     keyId = the custom key to drop from the allowed list.
	 */
	public void removeCustomAllowed(string keyId)
	{
		asc_compose_remove_custom_allowed(ascCompose, Str.toStringz(keyId));
	}

	/**
	 * Remove compose flags.
	 *
	 * Params:
	 *     flags = The compose flags to remove.
	 */
	public void removeFlags(AscComposeFlags flags)
	{
		asc_compose_remove_flags(ascCompose, flags);
	}

	/**
	 * Reset the results, units and run-specific settings so the
	 * instance can be reused for another metadata generation run.
	 */
	public void reset()
	{
		asc_compose_reset(ascCompose);
	}

	/**
	 * Process the registered units and generate collection metadata from
	 * found components.
	 *
	 * Params:
	 *     cancellable = a #GCancellable.
	 *
	 * Returns: The results, or %NULL on error
	 *
	 * Throws: GException on failure.
	 */
	public PtrArray run(Cancellable cancellable)
	{
		GError* err = null;

		auto __p = asc_compose_run(ascCompose, (cancellable is null) ? null : cancellable.getCancellableStruct(), &err);

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Set a CA file holding one or more certificates to verify peers with
	 * for download operations performed by this #AscCompose.
	 *
	 * Params:
	 *     cainfo = a valid file path
	 */
	public void setCainfo(string cainfo)
	{
		asc_compose_set_cainfo(ascCompose, Str.toStringz(cainfo));
	}

	/**
	 * Set an custom callback to be run when most of the metadata has been loaded,
	 * but no expensive operations (like downloads or icon rendering) have been done yet.
	 * This can be used to ignore unwanted components early on.
	 *
	 * The callback function may be called from any thread, so it needs to ensure thread safety on its own.
	 *
	 * Params:
	 *     func = the #AscCheckMetainfoLoadResultFn function to be called
	 *     userData = user data for @func
	 */
	public void setCheckMetadataEarlyFunc(AscCheckMetadataEarlyFn func, void* userData)
	{
		asc_compose_set_check_metadata_early_func(ascCompose, func, userData);
	}

	/**
	 * Set an output location where generated metadata should be saved.
	 * If this is set to %NULL, no metadata will be saved.
	 *
	 * Params:
	 *     dir = the metadata save location.
	 */
	public void setDataResultDir(string dir)
	{
		asc_compose_set_data_result_dir(ascCompose, Str.toStringz(dir));
	}

	/**
	 * Set a custom desktop-entry field localization functions to be run for specialized
	 * desktop-entry localization schemes such as used in Ubuntu.
	 *
	 * The callback function may be called from any thread, so it needs to ensure thread safety on its own.
	 *
	 * Params:
	 *     func = the #AscTranslateDesktopTextFn function to be called
	 *     userData = user data for @func
	 */
	public void setDesktopEntryL10nFunc(AscTranslateDesktopTextFn func, void* userData)
	{
		asc_compose_set_desktop_entry_l10n_func(ascCompose, func, userData);
	}

	/**
	 * Set compose flags bitfield that controls the enabled features
	 * for this #AscCompose.
	 *
	 * Params:
	 *     flags = The compose flags bitfield.
	 */
	public void setFlags(AscComposeFlags flags)
	{
		asc_compose_set_flags(ascCompose, flags);
	}

	/**
	 * Set the format kind of the collection metadata that we should generate.
	 *
	 * Params:
	 *     kind = The format, e.g. %AS_FORMAT_KIND_XML
	 */
	public void setFormat(AsFormatKind kind)
	{
		asc_compose_set_format(ascCompose, kind);
	}

	/**
	 * Set an output location for HTML reports of issues generated
	 * during a compose run.
	 *
	 * Params:
	 *     dir = the hints data directory.
	 */
	public void setHintsResultDir(string dir)
	{
		asc_compose_set_hints_result_dir(ascCompose, Str.toStringz(dir));
	}

	/**
	 * Set an icon policy object, overriding the existing one.
	 *
	 * Params:
	 *     policy = an #AscIconPolicy instance
	 */
	public void setIconPolicy(IconPolicy policy)
	{
		asc_compose_set_icon_policy(ascCompose, (policy is null) ? null : policy.getIconPolicyStruct());
	}

	/**
	 * Set an output location where plain icons for the processed metadata
	 * are stored.
	 *
	 * Params:
	 *     dir = the icon storage location.
	 */
	public void setIconsResultDir(string dir)
	{
		asc_compose_set_icons_result_dir(ascCompose, Str.toStringz(dir));
	}

	/**
	 * Set a specific unit that is used for fetching locale information.
	 * This may be useful in case a special language pack layout is used,
	 * but is generally not necessary to be set explicitly, as locale
	 * will be found in the unit where the metadata is by default.
	 *
	 * Params:
	 *     localeUnit = the unit used for locale processing.
	 */
	public void setLocaleUnit(Unit localeUnit)
	{
		asc_compose_set_locale_unit(ascCompose, (localeUnit is null) ? null : localeUnit.getUnitStruct());
	}

	/**
	 * Set the maximum size a screenshot video or image can have.
	 * A size < 0 may be set to allow unlimited sizes, setting a limit of 0
	 * will disable screenshot caching entirely.
	 *
	 * Params:
	 *     sizeBytes = maximum size of a screenshot image or video in bytes
	 */
	public void setMaxScreenshotSize(ptrdiff_t sizeBytes)
	{
		asc_compose_set_max_screenshot_size(ascCompose, sizeBytes);
	}

	/**
	 * Set the media base URL for the generated metadata. Can be %NULL.
	 *
	 * Params:
	 *     url = the media base URL.
	 */
	public void setMediaBaseurl(string url)
	{
		asc_compose_set_media_baseurl(ascCompose, Str.toStringz(url));
	}

	/**
	 * Set an output location to store media (screenshots, icons, ...) that
	 * will be served on a webserver via the URL set as media baseurl.
	 *
	 * Params:
	 *     dir = the media storage location.
	 */
	public void setMediaResultDir(string dir)
	{
		asc_compose_set_media_result_dir(ascCompose, Str.toStringz(dir));
	}

	/**
	 * Set the metadata origin field (e.g. "debian" or "flathub")
	 *
	 * Params:
	 *     origin = the origin.
	 */
	public void setOrigin(string origin)
	{
		asc_compose_set_origin(ascCompose, Str.toStringz(origin));
	}

	/**
	 * Set the directory prefix the to-be-processed units are using.
	 *
	 * Params:
	 *     prefix = a directory prefix, e.g. "/usr"
	 */
	public void setPrefix(string prefix)
	{
		asc_compose_set_prefix(ascCompose, Str.toStringz(prefix));
	}
}
