/*
 * Licensed under the GNU Lesser General Public License Version 3
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the license, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

// generated automatically - do not change


module ascompose.IconPolicyIter;

private import ascompose.IconPolicy;
private import ascompose.c.functions;
public  import ascompose.c.types;


/**
 * A #AscIconPolicyIter structure represents an iterator that can be used
 * to iterate over the icon sizes / policy entries of an #AscIconPolicy.
 * #AscIconPolicyIter structures are typically allocated on the stack and
 * then initialized with asc_icon_policy_iter_init().
 */
public class IconPolicyIter
{
	/** the main Gtk struct */
	protected AscIconPolicyIter* ascIconPolicyIter;
	protected bool ownedRef;

	/** Get the main Gtk struct */
	public AscIconPolicyIter* getIconPolicyIterStruct(bool transferOwnership = false)
	{
		if (transferOwnership)
			ownedRef = false;
		return ascIconPolicyIter;
	}

	/** the main Gtk struct as a void* */
	protected void* getStruct()
	{
		return cast(void*)ascIconPolicyIter;
	}

	/**
	 * Sets our main struct and passes it to the parent class.
	 */
	public this (AscIconPolicyIter* ascIconPolicyIter, bool ownedRef = false)
	{
		this.ascIconPolicyIter = ascIconPolicyIter;
		this.ownedRef = ownedRef;
	}

	/**
	 * Creates a new policy iterator.
	 */
	this()
	{
		this(new AscIconPolicyIter);
	}

	/**
	 */

	/**
	 * Initializes a policy iterator for the policy entry list and associates it
	 * it with @ipolicy.
	 * The #AscIconPolicyIter structure is typically allocated on the stack
	 * and does not need to be freed explicitly.
	 *
	 * Params:
	 *     ipolicy = an #AscIconPolicy
	 */
	public void init(IconPolicy ipolicy)
	{
		asc_icon_policy_iter_init(ascIconPolicyIter, (ipolicy is null) ? null : ipolicy.getIconPolicyStruct());
	}

	/**
	 * Returns the current icon policy entry and advances the iterator.
	 * Example:
	 * |[<!-- language="C" -->
	 * AscIconPolicyIter iter;
	 * guint icon_size;
	 * guint icon_scale;
	 * AscIconState istate;
	 *
	 * asc_icon_policy_iter_init (&iter, ipolicy);
	 * while (asc_icon_policy_iter_next (&iter, &icon_size, &icon_scale, &istate)) {
	 * // do something with the icon entry data
	 * }
	 * ]|
	 *
	 * Params:
	 *     size = Destination of the returned icon size
	 *     scale = Destination of the returned icon scale factor
	 *     state = Destination of the returned designated icon state.
	 *
	 * Returns: %FALSE if the last entry has been reached.
	 */
	public bool next(out uint size, out uint scale, out AscIconState state)
	{
		return asc_icon_policy_iter_next(ascIconPolicyIter, &size, &scale, &state) != 0;
	}
}
