/*
 * Licensed under the GNU Lesser General Public License Version 3
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the license, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

// generated automatically - do not change


module ascompose.Image;

private import ascompose.c.functions;
public  import ascompose.c.types;
private import gdkpixbuf.Pixbuf;
private import glib.ConstructionException;
private import glib.ErrorG;
private import glib.GException;
private import glib.HashTable;
private import glib.Str;
private import gobject.ObjectG;


/** */
public class Image : ObjectG
{
	/** the main Gtk struct */
	protected AscImage* ascImage;

	/** Get the main Gtk struct */
	public AscImage* getImageStruct(bool transferOwnership = false)
	{
		if (transferOwnership)
			ownedRef = false;
		return ascImage;
	}

	/** the main Gtk struct as a void* */
	protected override void* getStruct()
	{
		return cast(void*)ascImage;
	}

	/**
	 * Sets our main struct and passes it to the parent class.
	 */
	public this (AscImage* ascImage, bool ownedRef = false)
	{
		this.ascImage = ascImage;
		super(cast(GObject*)ascImage, ownedRef);
	}


	/** */
	public static GType getType()
	{
		return asc_image_get_type();
	}

	/**
	 * Creates a new #AscImage.
	 *
	 * Throws: ConstructionException GTK+ fails to create the object.
	 */
	public this()
	{
		auto __p = asc_image_new();

		if(__p is null)
		{
			throw new ConstructionException("null returned by new");
		}

		this(cast(AscImage*) __p, true);
	}

	/**
	 * Creates a new #AscImage from data in memory.
	 *
	 * Params:
	 *     data = Data to load.
	 *     len = Length of the data to load.
	 *     destSize = The size of the constructed pixbuf, or 0 for the native size
	 *     compressed = %TRUE if passed data is gzip-compressed
	 *     flags = a #AscImageLoadFlags, e.g. %ASC_IMAGE_LOAD_FLAG_NONE
	 *
	 * Throws: GException on failure.
	 * Throws: ConstructionException GTK+ fails to create the object.
	 */
	public this(void* data, ptrdiff_t len, uint destSize, bool compressed, AscImageLoadFlags flags)
	{
		GError* err = null;

		auto __p = asc_image_new_from_data(data, len, destSize, compressed, flags, &err);

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		if(__p is null)
		{
			throw new ConstructionException("null returned by new_from_data");
		}

		this(cast(AscImage*) __p, true);
	}

	/**
	 * Creates a new #AscImage from a file on the filesystem.
	 *
	 * Params:
	 *     fname = Name of the file to load.
	 *     destSize = The size of the constructed pixbuf, or 0 for the native size
	 *     flags = a #AscImageLoadFlags, e.g. %ASC_IMAGE_LOAD_FLAG_NONE
	 *
	 * Throws: GException on failure.
	 * Throws: ConstructionException GTK+ fails to create the object.
	 */
	public this(string fname, uint destSize, AscImageLoadFlags flags)
	{
		GError* err = null;

		auto __p = asc_image_new_from_file(Str.toStringz(fname), destSize, flags, &err);

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		if(__p is null)
		{
			throw new ConstructionException("null returned by new_from_file");
		}

		this(cast(AscImage*) __p, true);
	}

	/**
	 * Get a set of image format names we can currently read
	 * (via GdkPixbuf).
	 *
	 * Returns: A hash set of format names.
	 */
	public static HashTable supportedFormatNames()
	{
		auto __p = asc_image_supported_format_names();

		if(__p is null)
		{
			return null;
		}

		return new HashTable(cast(GHashTable*) __p, true);
	}

	/**
	 * Gets the image height.
	 */
	public uint getHeight()
	{
		return asc_image_get_height(ascImage);
	}

	/**
	 * Gets the image pixbuf if set.
	 *
	 * Returns: the #GdkPixbuf, or %NULL
	 */
	public Pixbuf getPixbuf()
	{
		auto __p = asc_image_get_pixbuf(ascImage);

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(Pixbuf)(cast(GdkPixbuf*) __p);
	}

	/**
	 * Gets the image width.
	 */
	public uint getWidth()
	{
		return asc_image_get_width(ascImage);
	}

	/**
	 * Reads an image from a file.
	 *
	 * Params:
	 *     filename = filename to read from
	 *     destSize = The size of the constructed pixbuf, or 0 for the native size
	 *     srcSizeMin = The smallest source size allowed, or 0 for none
	 *     flags = a #AscImageLoadFlags, e.g. %ASC_IMAGE_LOAD_FLAG_NONE
	 *
	 * Returns: %TRUE for success
	 *
	 * Throws: GException on failure.
	 */
	public bool loadFilename(string filename, uint destSize, uint srcSizeMin, AscImageLoadFlags flags)
	{
		GError* err = null;

		auto __p = asc_image_load_filename(ascImage, Str.toStringz(filename), destSize, srcSizeMin, flags, &err) != 0;

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		return __p;
	}

	/**
	 * Saves the image to a file.
	 *
	 * Params:
	 *     filename = filename to write to
	 *     width = target width, or 0 for default
	 *     height = target height, or 0 for default
	 *     flags = some #AscImageSaveFlags values, e.g. %ASC_IMAGE_SAVE_FLAG_PAD_16_9
	 *
	 * Returns: %TRUE for success
	 *
	 * Throws: GException on failure.
	 */
	public bool saveFilename(string filename, uint width, uint height, AscImageSaveFlags flags)
	{
		GError* err = null;

		auto __p = asc_image_save_filename(ascImage, Str.toStringz(filename), width, height, flags, &err) != 0;

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		return __p;
	}

	/**
	 * Resamples a pixbuf to a specific size.
	 *
	 * Params:
	 *     width = target width, or 0 for default
	 *     height = target height, or 0 for default
	 *     flags = some #AscImageSaveFlags values, e.g. %ASC_IMAGE_SAVE_FLAG_PAD_16_9
	 *
	 * Returns: A #GdkPixbuf of the specified size
	 */
	public Pixbuf savePixbuf(uint width, uint height, AscImageSaveFlags flags)
	{
		auto __p = asc_image_save_pixbuf(ascImage, width, height, flags);

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(Pixbuf)(cast(GdkPixbuf*) __p, true);
	}

	/**
	 * Scale the image to the given size.
	 *
	 * Params:
	 *     newWidth = The new width.
	 *     newHeight = the new height.
	 */
	public void scale(uint newWidth, uint newHeight)
	{
		asc_image_scale(ascImage, newWidth, newHeight);
	}

	/**
	 * Scale the image to fir in a square with the given edge length,
	 * and keep its aspect ratio.
	 *
	 * Params:
	 *     size = the maximum edge length.
	 */
	public void scaleToFit(uint size)
	{
		asc_image_scale_to_fit(ascImage, size);
	}

	/**
	 * Scale the image to the given height, preserving
	 * its aspect ratio.
	 *
	 * Params:
	 *     newHeight = the new height.
	 */
	public void scaleToHeight(uint newHeight)
	{
		asc_image_scale_to_height(ascImage, newHeight);
	}

	/**
	 * Scale the image to the given width, preserving
	 * its aspect ratio.
	 *
	 * Params:
	 *     newWidth = The new width.
	 */
	public void scaleToWidth(uint newWidth)
	{
		asc_image_scale_to_width(ascImage, newWidth);
	}

	/**
	 * Sets the image pixbuf.
	 *
	 * Params:
	 *     pixbuf = the #GdkPixbuf, or %NULL
	 */
	public void setPixbuf(Pixbuf pixbuf)
	{
		asc_image_set_pixbuf(ascImage, (pixbuf is null) ? null : pixbuf.getPixbufStruct());
	}
}
