/*
 * Licensed under the GNU Lesser General Public License Version 3
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the license, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

// generated automatically - do not change


module ascompose.Result;

private import appstream.Component;
private import ascompose.c.functions;
public  import ascompose.c.types;
private import glib.Bytes;
private import glib.ConstructionException;
private import glib.ErrorG;
private import glib.GException;
private import glib.PtrArray;
private import glib.Str;
private import glib.c.functions;
private import gobject.ObjectG;


/** */
public class Result : ObjectG
{
	/** the main Gtk struct */
	protected AscResult* ascResult;

	/** Get the main Gtk struct */
	public AscResult* getResultStruct(bool transferOwnership = false)
	{
		if (transferOwnership)
			ownedRef = false;
		return ascResult;
	}

	/** the main Gtk struct as a void* */
	protected override void* getStruct()
	{
		return cast(void*)ascResult;
	}

	/**
	 * Sets our main struct and passes it to the parent class.
	 */
	public this (AscResult* ascResult, bool ownedRef = false)
	{
		this.ascResult = ascResult;
		super(cast(GObject*)ascResult, ownedRef);
	}


	/** */
	public static GType getType()
	{
		return asc_result_get_type();
	}

	/**
	 * Creates a new #AscResult.
	 *
	 * Throws: ConstructionException GTK+ fails to create the object.
	 */
	public this()
	{
		auto __p = asc_result_new();

		if(__p is null)
		{
			throw new ConstructionException("null returned by new");
		}

		this(cast(AscResult*) __p, true);
	}

	/**
	 * Add component to the results set.
	 *
	 * Params:
	 *     cpt = The #AsComponent to add.
	 *     bytes = Source data used to generate the GCID hash, or %NULL if nonexistent.
	 *
	 * Returns: %TRUE on success.
	 *
	 * Throws: GException on failure.
	 */
	public bool addComponent(Component cpt, Bytes bytes)
	{
		GError* err = null;

		auto __p = asc_result_add_component(ascResult, (cpt is null) ? null : cpt.getComponentStruct(), (bytes is null) ? null : bytes.getBytesStruct(), &err) != 0;

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		return __p;
	}

	/**
	 * Add component to the results set, using string data.
	 *
	 * Params:
	 *     cpt = The #AsComponent to add.
	 *     data = Source data used to generate the GCID hash, or %NULL if nonexistent.
	 *
	 * Returns: %TRUE on success.
	 *
	 * Throws: GException on failure.
	 */
	public bool addComponentWithString(Component cpt, string data)
	{
		GError* err = null;

		auto __p = asc_result_add_component_with_string(ascResult, (cpt is null) ? null : cpt.getComponentStruct(), Str.toStringz(data), &err) != 0;

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		return __p;
	}

	/**
	 * Add an issue hint which does not have any variables to replace in its
	 * explanation text for a component.
	 *
	 * Params:
	 *     cpt = The affected #AsComponent
	 *     tag = AppStream Compose Issue hint tag
	 *
	 * Returns: %TRUE if the added hint did not cause the component to be invalidated.
	 */
	public bool addHintSimple(Component cpt, string tag)
	{
		return asc_result_add_hint_simple(ascResult, (cpt is null) ? null : cpt.getComponentStruct(), Str.toStringz(tag)) != 0;
	}

	/**
	 * Returns: The amount of components found for this unit.
	 */
	public uint componentsCount()
	{
		return asc_result_components_count(ascResult);
	}

	/**
	 * Gets all components this #AsResult instance contains.
	 *
	 * Returns: An array of #AsComponent
	 */
	public PtrArray fetchComponents()
	{
		auto __p = asc_result_fetch_components(ascResult);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Get a list of all hints for all components that are registered with this result.
	 *
	 * Returns: An array of #AscHint
	 */
	public PtrArray fetchHintsAll()
	{
		auto __p = asc_result_fetch_hints_all(ascResult);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Retrieve the global component-ID string for the given component-ID,
	 * as long as component with the given ID is registered with this #AscResult.
	 * Otherwise, %NULL is returned.
	 *
	 * Params:
	 *     cid = Component ID to look for.
	 */
	public string gcidForCid(string cid)
	{
		return Str.toString(asc_result_gcid_for_cid(ascResult, Str.toStringz(cid)));
	}

	/** */
	public string gcidForComponent(Component cpt)
	{
		return Str.toString(asc_result_gcid_for_component(ascResult, (cpt is null) ? null : cpt.getComponentStruct()));
	}

	/**
	 * Gets the ID name of the bundle (a package / Flatpak / any entity containing metadata)
	 * that these these results are generated for.
	 */
	public string getBundleId()
	{
		return Str.toString(asc_result_get_bundle_id(ascResult));
	}

	/**
	 * Gets the bundle kind these results are for.
	 */
	public AsBundleKind getBundleKind()
	{
		return asc_result_get_bundle_kind(ascResult);
	}

	/**
	 * Gets the component by its component-id-
	 *
	 * Params:
	 *     cid = Component ID to look for.
	 *
	 * Returns: An #AsComponent
	 */
	public Component getComponent(string cid)
	{
		auto __p = asc_result_get_component(ascResult, Str.toStringz(cid));

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(Component)(cast(AsComponent*) __p);
	}

	/**
	 * Retrieve a list of all global component-IDs that this result knows of.
	 *
	 * Returns: An array of global component IDs. Free with %g_free
	 */
	public string[] getComponentGcids()
	{
		auto retStr = asc_result_get_component_gcids(ascResult);

		scope(exit) g_free(retStr);
		return Str.toStringArray(retStr);
	}

	/**
	 * Gets list of component-IDs which do have issue hints associated with them.
	 *
	 * Returns: An array of component-IDs. Free container with %g_free
	 */
	public string[] getComponentIdsWithHints()
	{
		auto retStr = asc_result_get_component_ids_with_hints(ascResult);

		scope(exit) g_free(retStr);
		return Str.toStringArray(retStr);
	}

	/**
	 * Gets hints for a component with the given component-id.
	 *
	 * Params:
	 *     cid = Component ID to look for.
	 *
	 * Returns: An array of #AscHint or %NULL
	 */
	public PtrArray getHints(string cid)
	{
		auto __p = asc_result_get_hints(ascResult, Str.toStringz(cid));

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Test if a hint tag is associated with a given component in this result.
	 *
	 * Params:
	 *     cpt = the #AsComponent to check
	 *     tag = the hint tag to check for
	 *
	 * Returns: %TRUE if a hint with this tag exists for the selected component.
	 */
	public bool hasHint(Component cpt, string tag)
	{
		return asc_result_has_hint(ascResult, (cpt is null) ? null : cpt.getComponentStruct(), Str.toStringz(tag)) != 0;
	}

	/**
	 * Returns: The amount of hints emitted for this unit.
	 */
	public uint hintsCount()
	{
		return asc_result_hints_count(ascResult);
	}

	/**
	 * Check if an #AsComponent was set to be ignored in this result
	 * (usually due to errors).
	 *
	 * Params:
	 *     cpt = the component to check for.
	 *
	 * Returns: %TRUE if the component is ignored.
	 */
	public bool isIgnored(Component cpt)
	{
		return asc_result_is_ignored(ascResult, (cpt is null) ? null : cpt.getComponentStruct()) != 0;
	}

	/**
	 * Remove a component from the results set.
	 *
	 * Params:
	 *     cpt = The #AsComponent to remove.
	 *
	 * Returns: %TRUE if the component was found and removed.
	 */
	public bool removeComponent(Component cpt)
	{
		return asc_result_remove_component(ascResult, (cpt is null) ? null : cpt.getComponentStruct()) != 0;
	}

	/**
	 * Remove a component from the results set.
	 *
	 * Params:
	 *     cid = a component-ID
	 *
	 * Returns: %TRUE if the component was found and removed.
	 */
	public bool removeComponentById(string cid)
	{
		return asc_result_remove_component_by_id(ascResult, Str.toStringz(cid)) != 0;
	}

	/**
	 * Remove a component from the results set.
	 *
	 * Params:
	 *     cpt = The #AsComponent to remove.
	 *     removeGcid = %TRUE if global component ID should be unregistered as well.
	 *
	 * Returns: %TRUE if the component was found and removed.
	 */
	public bool removeComponentFull(Component cpt, bool removeGcid)
	{
		return asc_result_remove_component_full(ascResult, (cpt is null) ? null : cpt.getComponentStruct(), removeGcid) != 0;
	}

	/**
	 * Remove all hints that we have associated with the selected component-ID.
	 *
	 * Params:
	 *     cid = The component ID
	 */
	public void removeHintsForCid(string cid)
	{
		asc_result_remove_hints_for_cid(ascResult, Str.toStringz(cid));
	}

	/**
	 * Sets the name of the bundle these results are for.
	 *
	 * Params:
	 *     id = The new ID.
	 */
	public void setBundleId(string id)
	{
		asc_result_set_bundle_id(ascResult, Str.toStringz(id));
	}

	/**
	 * Sets the kind of the bundle these results are for.
	 */
	public void setBundleKind(AsBundleKind kind)
	{
		asc_result_set_bundle_kind(ascResult, kind);
	}

	/**
	 * Returns: %TRUE if this result means the analyzed unit was ignored entirely.
	 */
	public bool unitIgnored()
	{
		return asc_result_unit_ignored(ascResult) != 0;
	}

	/**
	 * Update the global component ID for the given component.
	 *
	 * Params:
	 *     cpt = The #AsComponent to edit.
	 *     bytes = The data to include in the global component ID, or %NULL
	 *
	 * Returns: %TRUE if the component existed and was updated.
	 */
	public bool updateComponentGcid(Component cpt, Bytes bytes)
	{
		return asc_result_update_component_gcid(ascResult, (cpt is null) ? null : cpt.getComponentStruct(), (bytes is null) ? null : bytes.getBytesStruct()) != 0;
	}

	/**
	 * Update the global component ID for the given component.
	 * This is a convenience method for %asc_result_update_component_gcid
	 *
	 * Params:
	 *     cpt = The #AsComponent to edit.
	 *     data = The data as string to include in the global component ID, or %NULL
	 *
	 * Returns: %TRUE if the component existed and was updated.
	 */
	public bool updateComponentGcidWithString(Component cpt, string data)
	{
		return asc_result_update_component_gcid_with_string(ascResult, (cpt is null) ? null : cpt.getComponentStruct(), Str.toStringz(data)) != 0;
	}

	/**
	 * Add an issue hint for a component.
	 *
	 * Params:
	 *     cpt = The affected #AsComponent
	 *     tag = AppStream Compose Issue hint tag
	 *     kv = List of key-value pairs for replacement variables.
	 *
	 * Returns: %TRUE if the added hint did not cause the component to be invalidated.
	 */
	public bool addHint(Component cpt, string tag, string[] kv)
	{
		return asc_result_add_hint_v(ascResult, (cpt is null) ? null : cpt.getComponentStruct(), Str.toStringz(tag), Str.toStringzArray(kv)) != 0;
	}

	/**
	 * Add an issue hint for a component.
	 *
	 * Params:
	 *     componentId = The component-ID of the affected #AsComponent
	 *     tag = AppStream Compose Issue hint tag
	 *     kv = List of key-value pairs for replacement variables.
	 *
	 * Returns: %TRUE if the added hint did not cause the component to be invalidated.
	 */
	public bool addHintByCid(string componentId, string tag, string[] kv)
	{
		return asc_result_add_hint_by_cid_v(ascResult, Str.toStringz(componentId), Str.toStringz(tag), Str.toStringzArray(kv)) != 0;
	}
}
