/*
 * Licensed under the GNU Lesser General Public License Version 3
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the license, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

// generated automatically - do not change


module ascompose.Unit;

private import ascompose.c.functions;
public  import ascompose.c.types;
private import glib.Bytes;
private import glib.ConstructionException;
private import glib.ErrorG;
private import glib.GException;
private import glib.PtrArray;
private import glib.Str;
private import glib.c.functions;
private import gobject.ObjectG;


/** */
public class Unit : ObjectG
{
	/** the main Gtk struct */
	protected AscUnit* ascUnit;

	/** Get the main Gtk struct */
	public AscUnit* getUnitStruct(bool transferOwnership = false)
	{
		if (transferOwnership)
			ownedRef = false;
		return ascUnit;
	}

	/** the main Gtk struct as a void* */
	protected override void* getStruct()
	{
		return cast(void*)ascUnit;
	}

	/**
	 * Sets our main struct and passes it to the parent class.
	 */
	public this (AscUnit* ascUnit, bool ownedRef = false)
	{
		this.ascUnit = ascUnit;
		super(cast(GObject*)ascUnit, ownedRef);
	}


	/** */
	public static GType getType()
	{
		return asc_unit_get_type();
	}

	/**
	 * Creates a new #AscUnit.
	 *
	 * Throws: ConstructionException GTK+ fails to create the object.
	 */
	public this()
	{
		auto __p = asc_unit_new();

		if(__p is null)
		{
			throw new ConstructionException("null returned by new");
		}

		this(cast(AscUnit*) __p, true);
	}

	/**
	 * Add a path to the list of relevant directories.
	 * A unit may only read data in paths that were previously
	 * registered as relevant.
	 *
	 * Params:
	 *     path = path to be considered relevant
	 */
	public void addRelevantPath(string path)
	{
		asc_unit_add_relevant_path(ascUnit, Str.toStringz(path));
	}

	/**
	 * Close this unit, possibly freeing its resources. Calls to read_data() or
	 * get_contents() may not produce results until open() is called again.
	 */
	public void close()
	{
		asc_unit_close(ascUnit);
	}

	/**
	 * Returns %TRUE if the directory exists and files in it are readable.
	 *
	 * Params:
	 *     dirname = The directory name to check.
	 */
	public bool dirExists(string dirname)
	{
		return asc_unit_dir_exists(ascUnit, Str.toStringz(dirname)) != 0;
	}

	/**
	 * Returns %TRUE if the filename exists and is readable using %asc_unit_read_data.
	 *
	 * Params:
	 *     filename = The filename to check.
	 */
	public bool fileExists(string filename)
	{
		return asc_unit_file_exists(ascUnit, Str.toStringz(filename)) != 0;
	}

	/**
	 * Gets the ID name of the bundle (a package / Flatpak / any entity containing metadata)
	 * that this unit represents.
	 */
	public string getBundleId()
	{
		return Str.toString(asc_unit_get_bundle_id(ascUnit));
	}

	/**
	 * Gets the ID name of the bundle, normalized to be safe to use
	 * in filenames. This may *not* be the same name as set via asc_unit_get_bundle_id()
	 */
	public string getBundleIdSafe()
	{
		return Str.toString(asc_unit_get_bundle_id_safe(ascUnit));
	}

	/**
	 * Gets the bundle kind of this unit.
	 */
	public AsBundleKind getBundleKind()
	{
		return asc_unit_get_bundle_kind(ascUnit);
	}

	/**
	 * Get a list of all files contained by this unit.
	 *
	 * Returns: A file listing
	 */
	public PtrArray getContents()
	{
		auto __p = asc_unit_get_contents(ascUnit);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Get a list of paths that are relevant for data processing.
	 *
	 * Returns: A list of paths
	 */
	public PtrArray getRelevantPaths()
	{
		auto __p = asc_unit_get_relevant_paths(ascUnit);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Get user-defined data. This is a helper
	 * function for bindings.
	 */
	public void* getUserData()
	{
		return asc_unit_get_user_data(ascUnit);
	}

	/**
	 * Open this unit, populating its content listing.
	 *
	 * Throws: GException on failure.
	 */
	public bool open()
	{
		GError* err = null;

		auto __p = asc_unit_open(ascUnit, &err) != 0;

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		return __p;
	}

	/**
	 * Read the contents of the selected file into memory and return them.
	 *
	 * Params:
	 *     filename = The file to read data for.
	 *
	 * Throws: GException on failure.
	 */
	public Bytes readData(string filename)
	{
		GError* err = null;

		auto __p = asc_unit_read_data(ascUnit, Str.toStringz(filename), &err);

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		if(__p is null)
		{
			return null;
		}

		return new Bytes(cast(GBytes*) __p, true);
	}

	/**
	 * Sets the ID of the bundle represented by this unit.
	 *
	 * Params:
	 *     id = The new ID.
	 */
	public void setBundleId(string id)
	{
		asc_unit_set_bundle_id(ascUnit, Str.toStringz(id));
	}

	/**
	 * Sets the kind of the bundle this unit represents.
	 */
	public void setBundleKind(AsBundleKind kind)
	{
		asc_unit_set_bundle_kind(ascUnit, kind);
	}

	/**
	 * Set list of files this unit contains.
	 *
	 * Params:
	 *     contents = A list of files contained by this unit.
	 */
	public void setContents(PtrArray contents)
	{
		asc_unit_set_contents(ascUnit, (contents is null) ? null : contents.getPtrArrayStruct());
	}

	/**
	 * Assign user-defined data to this object. This is a helper
	 * function for bindings.
	 *
	 * Params:
	 *     userData = the user data
	 */
	public void setUserData(void* userData)
	{
		asc_unit_set_user_data(ascUnit, userData);
	}
}
